/*!
 * Snake Game v 1.0
 * http://www.mikusa.com/
 *
 * Copyright 2011, Daniel Mikusa
 * Licensed under the GPL Version 2 license.
 * http://www.mikusa.com/snake/license
 *
 * Date: Sun Jan 30 11:03 2011 -0500
 */
var Frame = {
    lineWidth: 6,
    display: function() {
        // display border
        context.save();
        context.translate(this.border.left(), this.border.top());
        context.strokeStyle = "rgb(255,119,17)";
        context.lineWidth = this.lineWidth;
        context.strokeRect(0, 0, this.border.width(), this.border.height());
        context.restore();
    },
    clear: function() {
        context.save();
        context.fillStyle = "rgb(0,0,255)";
        context.fillRect(0, 0, board.width, board.height);
        context.restore();
    },
    gameOver: function() {
        this.displayMessage("Game Over!");
        this.displayHighScores();
    },
    preGame: function() {
    	this.displayMessage('Press "n" to start the game!');
    },
    displayMessage: function(message) {
    	this.clear();
    	context.save();
		context.translate(
                  (board.width / 4),
                  (board.height / 4));
        context.strokeStyle = "rgb(255,119,17)";
        context.lineWidth = this.lineWidth;
        context.strokeRect(0, 0, (board.width / 2), 50);
        context.fillStyle = "rgb(255,255,255)";
        context.font = "bold 16px Tahoma, Geneva, sans-serif";
        context.textAlign = "left";
        context.fillText(message, (board.width / 4) - (message.length * 4), 30);
        context.restore();    	
    },
    displayHighScores: function() {
        var scores = JSON.parse(localStorage["snake.scores"]);
    	context.save();
    	context.translate(
    				(board.width / 4),
    				(board.height / 4) * 1.75);
        context.strokeStyle = "rgb(255,119,17)";
		context.lineWidth = this.lineWidth;
        context.strokeRect(0, 0, (board.width / 2), 150);
        context.font = "bold 16px Tahoma, Geneva, sans-serif";
        context.textAlign = "left";
        for (i = 0; i < scores.length; i++) {
        	if (scores[i][2]) {
        		context.fillStyle = "rgb(255,255,0)";
        	} else {
        		context.fillStyle = "rgb(255,255,255)";
        	}
        	context.fillText(scores[i][0], 50, 30 + (i * 25));
        	context.fillText(scores[i][1], 200, 30 + (i * 25));
    	}
    	context.restore();
    },
    border: {
        cols: function() { return board.width / Block.WIDTH; },
        colRem: function() { return board.width % Block.WIDTH; },
        rows: function() { return board.height / Block.HEIGHT; },
        rowRem: function() { return board.height % Block.HEIGHT; },
        vline: function() { return (Frame.lineWidth / 2) + (Block.WIDTH - Frame.lineWidth); },
        hline: function() { return (Frame.lineWidth / 2) + (Block.HEIGHT - Frame.lineWidth); },
        left: function() {
            return (this.colRem() / 2) + Block.WIDTH + this.vline();
        },
        top: function() {
            return (this.rowRem() / 2) + (Block.HEIGHT * 3) + this.hline();
        },
        width: function() {
            return board.width - (this.left() * 2); 
        },
        height: function() {
            return board.height - (this.top() * 2) + 20;
        },
        right: function() {
            return this.left() + this.width();
        },
        bottom: function() {
            return this.top() + this.height();
        }
    }
}
