/*!
 * Snake Game v 1.0
 * http://www.mikusa.com/
 *
 * Copyright 2011, Daniel Mikusa
 * Licensed under the GPL Version 2 license.
 * http://www.mikusa.com/snake/license
 *
 * Date: Sun Jan 30 11:03 2011 -0500
 */
var Game = function(playerName) {
    this.init(playerName);
}
Game.speed = 30; // 30 frames per second
Game.counter = 0; // fps counter
Game.start = new Date().getTime(); // fps timer
Game.prototype.score = null;
Game.prototype.food = null;
Game.prototype.done = false;
Game.prototype.init = function(playerName) {
    this.score = new Score(playerName);
}
Game.prototype.randomlyPlaceFood = function() {
    var food = new Food(
    			   this.score.foodEaten + 1,
                   Math.floor(Math.random() * (Frame.border.width() - Block.WIDTH) / Block.WIDTH),
                   Math.floor(Math.random() * (Frame.border.height() - Block.HEIGHT) / Block.HEIGHT));
    while (! Snake.checkPositionOfNewFood(new Block(food.x, food.y))) {
        food = new Food(
        		   this.score.foodEaten + 1,
                   Math.floor(Math.random() * (Frame.border.width() - Block.WIDTH) / Block.WIDTH),
                   Math.floor(Math.random() * (Frame.border.height() - Block.HEIGHT) / Block.HEIGHT));
    }
    this.food = food;
}
Game.prototype.startGame = function() {
    if (this.timer != null) {
        this.stopTimer();
    }
    this.startTimer();
}
Game.prototype.togglePause = function() {
    if (this.timer == null) {
        this.startTimer();
    } else {    	
	    game.decrementScore(10);
        this.stopTimer();
    }
}
Game.prototype.startTimer = function() {
    this.timer = setTimeout("game.display()", Game.speed);
}
Game.prototype.stopTimer = function() {
    clearTimeout(this.timer);
    this.timer = null;
}
Game.prototype.resetTimer = function() {
    this.stopTimer();
    this.startTimer();
}
Game.prototype.display = function() {
    Game.counter += 1;
    var curMillis = new Date().getTime();
    var diff = (curMillis - Game.start);
    if (diff > 1000) {
        var fps = (Game.counter / diff * 1000);
        console.debug("time: " + diff + " counter: " + Game.counter + " fps: " + fps);
        Game.start = curMillis;
        Game.counter = 0;
    }
    Frame.display();
    this.score.display();
    this.food.display();
    if (! this.done) {
        //console.debug("Game not done, rescheduling display timer");
        this.startTimer();
    }
}
Game.prototype.incrementScore = function() {
	this.score.foodEaten += 1;
	this.score.points += Math.floor((1 / Snake.speed) * 250) * Snake.level;
}
Game.prototype.decrementScore = function(amount) {
	this.score.points -= amount;
	this.score.display();
}
Game.debugScores = function() {
    console.debug("scores deleted");
    localStorage.removeItem("snake.scores");
}
Game.clearScores = function() {
	//console.info("Scores reset");
	localStorage["snake.scores"] = JSON.stringify([
										["Bob", 250, false],
										["Frank", 200, false],
										["William", 150, false],
										["Jane", 100, false],
										["Sarah", 50, false]]);
}
Game.prototype.saveScore = function() {
    var scoreStr = localStorage["snake.scores"];
    if (scoreStr == null || scoreStr == "") {
        Game.clearScores();
        scoreStr = localStorage["snake.scores"];
    }
    var scores = JSON.parse(localStorage["snake.scores"]);
    for (i = 0; i < scores.length; i++) {
        scores[i][2] = false;
    }
    for (i = 0; i < scores.length; i++) {
		if (this.score.points >= scores[i][1]) {
			scores.splice(i, 0, [this.score.playerName, this.score.points, true]);
			scores.pop();
			break;
		}
	}
	localStorage["snake.scores"] = JSON.stringify(scores);
}
