/*!
 * Snake Game v 1.0
 * http://www.mikusa.com/
 *
 * Copyright 2011, Daniel Mikusa
 * Licensed under the GPL Version 2 license.
 * http://www.mikusa.com/snake/license
 *
 * Date: Sun Jan 30 11:03 2011 -0500
 */
var Snake = {
    blocks: null,
    direction: 0,
    speed: 1000,
    timer: null,
    growSnake: null,
    level: null,

    init: function(size) {
        this.blocks = new Array();
        size = (size) ? size : 10;
        var centerX = board.width / 2;
        var centerY = board.height / 2;
        for (i = 0; i < size; i++) {
            this.blocks.push(new Block(centerX, centerY, i))
        }
        if (this.timer != null) {
            this.clearTimer();
        }
        this.setTimer();
    },
    
    setLevel: function(level) {
    	this.level = parseInt(level);
    	switch (this.level) {
    		case 1:
    			this.speed = 250;
    			this.growSnake = this.growSnakeTemplate(this['growSnakeLinear']);
    			break;
    		case 2:
    			this.speed = 125;
    			this.growSnake = this.growSnakeTemplate(this['growSnakeLinear']);
    			break;
    		case 3:
    			this.speed = 100;
    			this.growSnake = this.growSnakeTemplate(this['growSnakeRandom']);
    			break;
    		case 4:
    			this.speed = 75;
    			this.growSnake = this.growSnakeTemplate(this['growSnakeQuadratic']);
    			break;
    		case 5:
    			this.speed = 50;
    			this.growSnake = this.growSnakeTemplate(this['growSnakeExponential']);
    			break;
    		default:
    			this.level = 2;
    			this.speed = 125;
    			this.growSnake = this.growSnakeTemplate(this['growSnakeLinear']);
    			break;
    	}
    },

    setTimer: function() {
        this.timer = setTimeout("Snake.updatePosition()", Snake.speed);
    },

    resetTimer: function() {
        this.clearTimer();
        this.setTimer();
    },

    clearTimer: function() {
        clearTimeout(this.timer);
        this.timer = null;
    },

    togglePause: function() {
        if (this.timer == null) {
            this.setTimer();
        } else {
            this.clearTimer();
        }
    },

    display: function() {
        this.blocks[0].display();
    },

    clear: function() {
        this.blocks[this.blocks.length - 1].clear();
    },

    move: function(direction) {
        if (this.validateMove(direction)) {
            this.direction = direction;
        }
    },

    updatePosition: function() {
        this.clear();
        this.moveLastToFront();
        if (this.checkCollisions()) {
            this.handleCollision();
        } else {
            this.display();
            this.checkCollisionWithFood();
        }
        if (! game.done) {
            //console.debug("Game still running, rescheduling snake timer");
            this.setTimer();
        }
    },

    checkCollisions: function() {
        var collision = false;
        // check inside bounds of board
        collision = ! this.blocks[0].insideFrame();
        // check collision with self
        for (i = 1; i < this.blocks.length; i++) {
            collision = collision || this.blocks[0].collideWithBlock(this.blocks[i]);
        }
        return collision;
    },

    checkCollisionWithFood: function() {
        if (this.blocks[0].collideWithBlock(game.food)) {
            this.growSnake();
            game.incrementScore();
            game.randomlyPlaceFood();
        }
    },

    checkPositionOfNewFood: function(food) {
        var collision = false;
    	var tmp = "";
        for (i = 0; i < this.blocks.length; i++) {
        	tmp += this.blocks[i].debug() + ", ";
        	collision = collision || this.blocks[i].collideWithBlock(food);
        }
        //console.debug("snake: [" + tmp + "]");
        //console.debug("collision with food: " + collision + " [" + food.x + "," + food.y + "]");
        return ! collision;
    },

    moveLastToFront: function() {
        var last = this.blocks.pop();
        var first = this.blocks[0];
        last.x = first.x;
        last.y = first.y;
        last.updatePosition(this.direction);
        this.blocks.unshift(last);
    },

    validateMove: function(direction) {
        if (this.blocks[0].collideWithBlock(this.blocks[1]) &&
                this.blocks[0].collideWithBlock(this.blocks[2]))
        {
            return true;
        } else {
            var first = this.blocks[0];
            var tmp = new Block(first.x, first.y, first.id);
            tmp.updatePosition(direction);
            return ! tmp.collideWithBlock(this.blocks[1]);
        }
    },

    handleCollision: function() {
        this.clearTimer();
        game.stopTimer();
        game.done = true;
        game.saveScore();
        setTimeout("Frame.gameOver()", 100);
    },

    growSnakeTemplate: function(growth) {
        return function() {
            var last = this.blocks[this.blocks.length - 1];
            var inc = growth();
            //console.debug("Growing snake by [" + inc + "]");
            for (var i = 0; i < inc; i++) {
                this.blocks.push(new Block(last.x, last.y, last.id + 1));
            }
        }
    },

    growSnakeLinear: function() { return 2; },
    growSnakeQuadratic: function() { return Math.pow(game.score.foodEaten, 2); },
    growSnakeRandom: function() { return Math.floor(Math.random() * 10) + 1; },
    growSnakeExponential: function() { return Math.pow(2, game.score.foodEaten); }
};

